(*

  application: image filtering
  state: experimental
  revision: $Id: defs.ml,v 1.25 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

type point = { redChannel:int; greenChannel:int; blueChannel:int }
 
exception Type_error of string
exception Lookup_error of string
type value  = 
    VInt of int
  | VFloat of float
  | VBool of bool
  | VMatrix of float array array


type 'a maybe = Just of 'a | Nothing

type dtype = Bool | Int | Float | Matrix

type color = Red | Green | Blue

type unOp 
 = NegI | NegF
 | Not
 | AbsI | AbsF
 | Exp | Log | Sqrt | Sin | Cos | Tan | ASin | ACos | ATan

type binOp 
 = AddI | SubI | MulI | DivI | PowI | MinI | MaxI | ModI  
 | AddF | SubF | MulF | DivF | PowF | MinF | MaxF
 | EqB  | UEqB | AndB | OrB
 | EqI  | UEqI | LtI  | LeI  | GtI  | GeI
 | EqF  | UEqF | LtF  | LeF  | GtF  | GeF

type op = 
    C of value 
  | V of string
  | Read of color        
  | Int2Float 
  | Floor
  | UnOp  of unOp
  | BinOp of binOp        
  | If                    
  | Let of string         
  | IndexMatrix of string 
  | Sum of string        

type exp = Node of (dtype * op * exp list)

type 'a environment = string -> 'a option

let string_of_dtype t = match t with
   Int -> "Int"
 | Float -> "Float"
 | Bool -> "Bool"
 | Matrix -> "Matrix"
 
let extEnv (k,v) env s = if s=k then Some v else env s

let emptyEnv s = None

let lookup n env = env n 

let lookup_excp n env = 
  match lookup n env with
  | Some v -> v
  | None -> raise (Lookup_error ("symbol "^n^" not in environment"))

let dtypeof (Node (t,_,_)) = t 

let unC = function
  | (Node (_,C c,_)) -> Some c
  | _               -> None


(* some helpers... *)
let rec fromto a b = if a>b then [] else a :: fromto (a+1) b

let head = function 
  | [a] -> a 
  | _ -> raise (Failure
      "in 'head': Wrong # of many arguments. Bug in Parser?");;
  
let head2 = function 
  | [a;b] -> (a,b) 
  | _ -> raise (Failure
      "in 'head2': Wrong # of many arguments. Bug in Parser?");;

let head3 = function 
  | [a;b;c] -> (a,b,c) 
  | _ -> raise (Failure
      "in 'head3': Wrong # of many arguments. Bug in Parser?");;

