(*

  application: image filtering
  state: experimental
  revision: $Id: defs.mli,v 1.22 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

exception Lookup_error of string
exception Type_error of string

type value  = 
    VInt of int
  | VFloat of float
  | VBool of bool
  | VMatrix of float array array


type dtype = Bool | Int | Float | Matrix

type color = Red | Green | Blue

type unOp 
 = NegI | NegF
 | Not
 | AbsI | AbsF
 | Exp | Log | Sqrt | Sin | Cos | Tan | ASin | ACos | ATan

type binOp 
 = AddI | SubI | MulI | DivI | PowI | MinI | MaxI | ModI  
 | AddF | SubF | MulF | DivF | PowF | MinF | MaxF
 | EqB  | UEqB | AndB | OrB
 | EqI  | UEqI | LtI  | LeI  | GtI  | GeI
 | EqF  | UEqF | LtF  | LeF  | GtF  | GeF


type op = 
    C of value 
  | V of string
  | Read of color         (* x:int,   y:int      *)
  | Int2Float 
  | Floor
  | UnOp of unOp
  | BinOp of binOp        
  | If                    (* x:bool, y:'a, z:'a *)
  | Let of string         (* x:'a,   y:'b       *)
  | IndexMatrix of string (* x:int,  y: int     *)
  | Sum of string        (* low:int, high:int, exp:int *)

type exp = Node of (dtype * op * exp list)

type 'a environment = string -> 'a option

val extEnv : string * 'a -> 'a environment -> 'a environment
val emptyEnv : 'a environment 
val lookup : string -> 'a environment -> 'a option
val lookup_excp : string -> 'a environment -> 'a
val dtypeof : exp -> dtype
val unC : exp -> value option
val fromto : int -> int -> int list
val head : 'a list -> 'a
val head2 : 'a list -> 'a * 'a
val head3 : 'a list -> 'a * 'a * 'a

