/*

  application: image filtering
  state: experimental
  revision: $Id: filt_parser.mly,v 1.9 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

%{ 
 open Defs 

 (*          (bool oper., int oper.,  float oper., intented type)  *)
 let addop = (None,       Some AddI,  Some AddF,   None) 
 let subop = (None,       Some SubI,  Some SubF,   None) 
 let mulop = (None,       Some MulI,  Some MulF,   None)
 let powop = (None,       None,       Some PowF,   None)
 let divop = (None,       Some DivI,  Some DivF,   None)
 let modop = (None,       Some ModI,  None,        None) 
 let andop = (Some AndB,  None,       None,        None)
 let orop  = (Some OrB,   None,       None,        None) 
 let minop = (None,       Some MinI,  Some MaxF,   None)
 let maxop = (None,       Some MaxI,  Some MaxF,   None)
 let eqop  = (Some EqB,   Some EqI,   Some EqF,    Some Bool) 
 let neqop = (Some UEqB,  Some UEqI,  Some UEqF,   Some Bool) 
 let ltop  = (None,       Some LtI,   Some LtF,    Some Bool) 
 let leop  = (None,       Some LeI,   Some LeF,    Some Bool) 
 let gtop  = (None,       Some GtI,   Some GtF,    Some Bool)
 let geop  = (None,       Some GeI,   Some GeF,    Some Bool)

 let notop    = (Some (UnOp Not), None, None, Some Bool)
 let uminusop = (None, Some (UnOp NegI), Some (UnOp NegF), None)
 let floorop  = (None, None, Some Floor, Some Int)
 let expop    = (None, None, Some (UnOp Exp), Some Float)
 let sinop    = (None, None, Some (UnOp Sin), Some Float)
 let cosop    = (None, None, Some (UnOp Cos), Some Float)
 let tanop    = (None, None, Some (UnOp Tan), Some Float)
 let logop    = (None, None, Some (UnOp Log), Some Float)
 let sqrtop   = (None, None, Some (UnOp Sqrt), Some Float)
 let absop    = (None, Some (UnOp AbsI), Some (UnOp AbsF), None)

 (* val listMatrix : float list list -> float array array *)
 let listMatrix xss = begin
   let len = 
     List.fold_left (fun x l -> max x (List.length l)) (List.length xss) xss 
   in
   let val_for_column i j  =
     if j>=List.length (List.nth xss i) then 
       0.0
     else 
       List.nth (List.nth xss i) j
   in
   let array_for_row i = 
     if i>=List.length xss then 
       Array.make len 0.0
     else 
       Array.init len (val_for_column i)
   in
   Array.init len array_for_row
 end
 
 (* val upcast_int_to_float : exp -> exp *)
 let upcast_int_to_float (exp:exp) = 
     match dtypeof exp with
       Int -> Node (Float,Int2Float,[exp])
     | Float -> exp
     | _ -> raise (Type_error "<<upcast_binding_time>>")

 (* val maybe : 'a option -> 'a -> 'a *)
 let maybe ma b = 
   match ma with 
     Some a  -> a
   | None -> b

 (* val infer_binop :
      binOp option * binOp option * binOp option *
      dtype option -> exp * exp -> exp  *)
 let infer_binop ops (e1,e2) =
   let t1,t2 = dtypeof e1,dtypeof e2 in
   let (boolOp,intOp,floatOp,t) = ops in
   let restype = maybe t t1 in
   match (t1,t2,boolOp,intOp,floatOp) with 
     (Bool, Bool,  Some op,_,_) -> Node (restype,BinOp op,[e1;e2])
   | (Int,  Int,   _,Some op,_) -> Node (restype,BinOp op,[e1;e2])
   | (Float,Float, _,_,Some op) -> Node (restype,BinOp op,[e1;e2])
   | (Int,  Float, _,_,Some op) -> 
       Node (maybe t Float, BinOp op,[ upcast_int_to_float e1; e2 ])
   | (Float,Int,_,_,Some op) -> 
       Node (maybe t Float, BinOp op,[ e1; upcast_int_to_float e2 ])
   | (Int,Int,_,_,Some op) -> 
       Node (maybe t Float, BinOp op,List.map upcast_int_to_float [ e1; e2 ])
   | _                 ->
         raise (Type_error ("can't infer binary operator: illegal operand type ."))

 (* val infer_unop :
      op option * op option * op option * dtype option -> exp -> exp *)
 let infer_unop ops exp =
   let ty = dtypeof exp in
   let (boolOp,intOp,floatOp,t) = ops in
   let restype = maybe t ty in
   match (ty,boolOp,intOp,floatOp) with
      (Bool, Some op,_,_) -> Node (restype,op,[exp])
    | (Int,  _,Some op,_) -> Node (restype,op,[exp])
    | (Float,  _,_,Some op) -> Node (restype,op,[exp])
    | (Int,  _,_,Some op) -> Node (Float,op,[upcast_int_to_float exp])
    | _                 ->
         raise (Type_error ("can't infer unary operator: illegal operand type ."))
    

%}

%token <int> INT
%token <float> FLOAT
%token <string> VAR
%token PLUS MINUS
%token TIMES DIV POWER
%token LPAREN RPAREN
%token LBRACK RBRACK
%token RED GREEN BLUE 
%token IF THEN ELSE
%token LET IN
%token SUM FROM TO OF
%token AND
%token OR
%token MOD
%token FLOOR SIN COS TAN EXP MAX MIN LOG SQRT ABS
%token NOT
%token EQ UEQ LESS LESSEQ GREATER GREATEQ
%token COMMA SEP
%token COLON SUM FROMINT
%token EOL
%token TRUE FALSE
%nonassoc THEN
%nonassoc ELSE
%nonassoc IN
%nonassoc TO
%nonassoc OF
%left OR
%left AND
%left EQ UEQ LESS LESSEQ GREATER GREATEQ
%left PLUS MINUS        /* lowest precedence */
%left TIMES DIV MOD     /* medium precedence */
%right POWER
%right UMINUS 
%start main             /* the entry point */
%type <Defs.exp> main 
%type <dtype environment -> Defs.exp> baseExp
%type <float list list> rows
%type <float list> elems
%type <color> readOp
%type <(op option * op option * op option * dtype option)> unaryOp
%type <float> mconst
%%
main: baseExp EOL { let env = emptyEnv in
                   let env = extEnv ("row",Int) env in
                   let env = extEnv ("col",Int) env in
                   let env = extEnv ("iter",Int) env in
                   let env = extEnv ("width",Int) env in
                   let env = extEnv ("height",Int) env in
                   let env = extEnv ("maxval",Int) env in
                   $1 env
                 }
;
readOp: RED      { Red   }
      | GREEN    { Green }
      | BLUE     { Blue  }
;

unaryOp:  EXP    { expop }
        | LOG    { logop }
        | ABS    { absop }
        | SQRT   { sqrtop }
        | SIN    { sinop }
        | COS    { cosop }
        | TAN    { tanop }
        | NOT    { notop }
        | MINUS  { uminusop }
        | FLOOR  { floorop }
;
baseExp:
    readOp LPAREN baseExp COMMA baseExp RPAREN 
          { fun env -> 
              let (op,x,y) = ($1,$3 env,$5 env) in
              if not (dtypeof x = Int && dtypeof y = Int) then
                begin
                  raise (Type_error 
                    ("Index of red/green/blue(..) is not Integer. "^
                     "Use floor(f) for explicit cast."))
                end
              else Node (Float,Read op,[x;y])
          }
  | TRUE  { 
      let null = Node (Int,  C (VInt 0),  []) in
      (fun _ -> Node (Bool, BinOp EqI,[null;null]))
  }
  | FALSE { 
      let null = Node (Int,  C (VInt 0),  []) in
      (fun _ -> Node (Bool, BinOp UEqI,[null;null]))
  }
  | FLOAT { fun _ -> Node (Float,C (VFloat $1),[]) }
  | INT   { fun _ -> Node (Int,  C (VInt $1),  []) }
  | VAR   { fun env -> 
              match lookup $1 env with
                 Some Matrix -> 
                   raise (Type_error 
                          ("Variable "^$1
                           ^" is of type Matrix but is used like a "
                           ^"basic type."))
               | Some t -> Node (t, V $1,[])
               | None -> raise (Type_error ("Variable "^ $1
                                   ^" is unknown here."))
          }

  | VAR LBRACK baseExp COMMA baseExp RBRACK 
          {
            fun env ->
              let ty, x, y = lookup $1 env, $3 env, $5 env in
              match (ty, dtypeof x,dtypeof y) with
                 (Some Matrix,Int,Int) -> 
                   Node (Float,IndexMatrix $1,[x;y])
               | (None, _,_) -> 
                   raise (Type_error ("Matrix "^ $1
                                   ^" is unknown here."))
               | _ ->
                   raise 
                      (Type_error ("At least one index of matrix "^$1
                       ^"(..) is not Integer."))
          }

  | LPAREN baseExp RPAREN
          { $2 }

  | baseExp PLUS baseExp
          { fun env -> infer_binop addop ($1 env, $3 env) }
  | baseExp MINUS baseExp
          { fun env -> infer_binop subop ($1 env, $3 env) }
  | baseExp TIMES baseExp
          { fun env -> infer_binop mulop ($1 env, $3 env) }
  | baseExp DIV baseExp
          { fun env -> infer_binop divop ($1 env, $3 env) }
  | baseExp MOD baseExp
          { fun env -> infer_binop modop ($1 env, $3 env) }
  | baseExp POWER baseExp
          { fun env -> infer_binop powop ($1 env, $3 env) }
  | baseExp AND  baseExp
          { fun env -> infer_binop andop ($1 env, $3 env) }
  | baseExp OR baseExp
          { fun env -> infer_binop orop ($1 env, $3 env) }
  | baseExp EQ baseExp
          { fun env -> infer_binop eqop ($1 env, $3 env) }
  | baseExp UEQ baseExp
          { fun env -> infer_binop neqop ($1 env, $3 env) }
  | baseExp LESS baseExp
          { fun env -> infer_binop ltop ($1 env, $3 env) }
  | baseExp LESSEQ baseExp
          { fun env -> infer_binop leop ($1 env, $3 env) }
  | baseExp GREATER baseExp
          { fun env -> infer_binop gtop ($3 env, $1 env) }
  | baseExp GREATEQ baseExp
          { fun env -> infer_binop geop ($3 env, $1 env) }
  | MAX LPAREN baseExp COMMA baseExp RPAREN
          { fun env -> infer_binop maxop ($3 env, $5 env) }
  | MIN LPAREN baseExp COMMA baseExp RPAREN
          { fun env -> infer_binop minop ($3 env, $5 env) }

  | unaryOp baseExp %prec UMINUS { fun env -> infer_unop $1 ($2 env) }


  | IF baseExp THEN baseExp ELSE baseExp
          { fun env -> 
              let cond,a,b =  $2 env, $4 env, $6 env in
              let (ct,at,bt) = head3 (List.map dtypeof [cond;a;b]) in
              if ct <> Bool then
                raise (Type_error "Condition not boolean in 'if'.")
              else if at <> bt then
                match (at,bt) with
                  (Int,Float) ->
                    Node (Float,If,[cond;upcast_int_to_float a;b])
                | (Float,Int) ->
                    Node (Float,If,[cond;a;upcast_int_to_float b])
                | _ ->
                raise (Type_error "Different body types in 'if'.")
              else
                 Node (at,If,[cond;a;b])
          }
  | LET VAR EQ baseExp IN baseExp 
    {
      fun env ->
        let var,rhs = $2, $4 env in
        let env' = extEnv (var, dtypeof rhs) env in
        let body = $6 env' in
        Node (dtypeof body, Let var,[rhs;body])
    }

  | SUM VAR FROM baseExp TO baseExp OF baseExp
          {
            fun env -> 
              let (upper,lower) = ($4 env, $6 env) in
              let env' = extEnv ($2, Int) env in
              let body = $8 env' in
              if not (dtypeof upper=Int && dtypeof lower = Int) then
                raise (Type_error "Sum has non-Integer Bound.")
              else
                let t = dtypeof body
                in Node (t,Sum $2,[upper;lower;body])
          }

  | LBRACK rows RBRACK
          {
            fun env -> 
              let m = listMatrix $2 in
              Node (Matrix,C (VMatrix m),[])
          }

;

rows:
      elems           { [ $1 ] }
    | elems SEP rows  { $1 :: $3 }
;

mconst: FLOAT   { $1 }
    | MINUS FLOAT { -.$2 }
;

elems:
      mconst           { [ $1 ] }
    | mconst elems     { $1 :: $2 }
;
%%


