(*

  application: image filtering
  state: experimental
  revision: $Id: ppm.mli,v 1.8 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

type point = { redChannel:int; greenChannel:int; blueChannel:int }

type ppm_image = 
  {
    height: int; 
    width: int ; 
    maxval: int; 
    raster: (point array) array }

val black : point

val gray_of : int -> point
val point_of_array : int array -> point
val create : int -> int -> int -> point -> ppm_image
val init : int -> int -> int -> (int -> int->point) -> ppm_image
val string_of_point : point -> string
val string_of_ppm_image : ppm_image -> string
val get_point : ppm_image -> int -> int -> point
val get_point_red : ppm_image -> int -> int -> int
val get_point_green : ppm_image -> int -> int -> int
val get_point_blue : ppm_image -> int -> int -> int
val set_point : ppm_image -> int -> int -> point -> unit

(* returns (height,width,maxval) *)
val get_dims: ppm_image -> int * int * int

exception UnexpectedInput of string

val input_ppm : in_channel -> ppm_image
val output_ppm : out_channel -> ppm_image -> unit

val load_ppm_file : string -> ppm_image
val store_ppm_file : string -> ppm_image -> unit



