(*

  application: image filtering
  state: experimental
  revision: $Id: stagedinter.ml,v 1.11 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

(*
sed command for stripping MetaOCaml syntax from this file

sed -e "s/>\./)/g;s/\.</(/g;s/\.[!~]/ /g"\
    -e "s/C\(Bool\|Float\|Int\)/V\1/g"\
    -e "s/^.*__only_staged__.*$//"

*)

open Defs 
open Ppm


type 'a codevalue =              (* __only_staged__ *)
    CInt of ('a,int) code        (* __only_staged__ *)
  | CFloat of ('a,float) code    (* __only_staged__ *)
  | CBool of ('a,bool) code      (* __only_staged__ *)
  | CMatrix of ('a,float array array) code      (* __only_staged__ *)

exception Inter_error of string
      

(* val unopfunCode : unOp -> 'a codevalue -> 'a codevalue *)
let unopfunCode op a =
  match (op,a) with
  | (NegI, CInt a)   -> CInt .< 0 - .~a >.
  | (NegF, CFloat a) -> CFloat .< 0.0 -. .~a >. 
  | (Not,  CBool a)  -> CBool .< not .~a >.
  | (AbsI, CInt a) -> CInt .< abs .~a >.
  | (AbsF, CFloat a) -> CFloat .< if .~a < 0.0 then -. .~a else .~a>.
  | (Exp, CFloat a) -> CFloat .< exp .~a >.
  | (Log, CFloat a) -> CFloat .< log .~a >.
  | (Sqrt, CFloat a) -> CFloat .< sqrt .~a >.
  | (Sin, CFloat a) -> CFloat .< sin .~a >.
  | (Cos, CFloat a) -> CFloat .< cos .~a >.
  | (Tan, CFloat a) -> CFloat .< tan .~a >.
  | (ASin, CFloat a) -> CFloat .< asin .~a >.
  | (ACos, CFloat a) -> CFloat .< acos .~a >.
  | (ATan, CFloat a) -> CFloat .< atan .~a >.
  | _ -> raise (Inter_error 
     "in function 'unopfunCode': Cannot create code for given operand.")


(* val binopfunCode : binOp -> 'a codevalue -> 'a codevalue 
                                            -> 'a codevalue *)
let binopfunCode op a b = match (op,a,b) with
  | (AddI, CInt   a, CInt   b) -> CInt   .< .~a  +  .~b >.
  | (SubI, CInt   a, CInt   b) -> CInt   .< .~a  -  .~b >.
  | (AddF, CFloat a, CFloat b) -> CFloat .< .~a  +. .~b >.
  | (EqI,  CInt   a, CInt   b) -> CBool  .< .~a  =  .~b >.
  | (MulI, CInt a, CInt b) -> CInt .< .~a * .~b >.
  | (DivI, CInt a, CInt b) -> CInt .< .~a / .~b >.
  | (ModI, CInt a, CInt b) -> CInt .< .~a mod .~b >.
  | (SubF, CFloat a, CFloat b) -> CFloat .< .~a -. .~b >.
  | (MulF, CFloat a, CFloat b) -> CFloat .< .~a *. .~b >.
  | (DivF, CFloat a, CFloat b) -> CFloat .< .~a /. .~b >.
  | (PowF, CFloat a, CFloat b) -> CFloat .< .~a ** .~b >.
  | (EqB, CBool a, CBool b) -> CBool .< .~a = .~b >.
  | (UEqB, CBool a, CBool b) -> CBool .< .~a != .~b >.
  | (AndB, CBool a, CBool b) -> CBool .< .~a && .~b >.
  | (OrB, CBool a, CBool b) -> CBool .< .~a || .~b >.
  | (MinI, CInt a, CInt b) -> CInt .< min .~a .~b >.
  | (MaxI, CInt a, CInt b) -> CInt .< max .~a .~b >.
  | (MinF, CFloat a, CFloat b) -> CFloat .< min .~a .~b  >.
  | (MaxF, CFloat a, CFloat b) -> CFloat .< max .~a .~b  >.
  | (UEqI, CInt a, CInt b) -> CBool .< .~a != .~b >.
  | (LtI, CInt a, CInt b) -> CBool .< .~a < .~b >.
  | (LeI, CInt a, CInt b) -> CBool .< .~a <= .~b >.
  | (GtI, CInt a, CInt b) -> CBool .< .~a > .~b >.
  | (GeI, CInt a, CInt b) -> CBool .< .~a >= .~b >.
  | (EqF, CFloat a, CFloat b) -> CBool .< .~a = .~b >.
  | (UEqF, CFloat a, CFloat b) -> CBool .< .~a != .~b >.
  | (LtF, CFloat a, CFloat b) -> CBool .< .~a < .~b >.
  | (LeF, CFloat a, CFloat b) -> CBool .< .~a <= .~b >.
  | (GtF, CFloat a, CFloat b) -> CBool .< .~a > .~b >.
  | (GeF, CFloat a, CFloat b) -> CBool .< .~a >= .~b >.
  | _ -> raise (Inter_error 
     "in function 'binopfunCode': Cannot create code for given operands.")


let unCBool = function 
  | (CBool i) -> i
  | _ -> 
      raise (Inter_error "in funtion 'unCBool': Value is not bool.")
let unCInt = function 
  | (CInt i) -> i
  | _ -> 
      raise (Inter_error "in funtion 'unCInt': Value is not int.")
let unCFloat = function
  | (CFloat f) -> f
  | _ -> raise (Inter_error 
     "in funtion 'unCFloat': Value is not float.")
let unCMatrix = function
  | (CMatrix f) -> f
  | _ -> raise (Inter_error 
     "in funtion 'unCMatrix': Value is not a matrix.")

let saveUnCFloat = function 
  | (CFloat f) -> f
  | (CInt i) -> .< float_of_int .~i >.
  | _ -> raise (Inter_error 
     "in funtion 'saveUnCFloat': Value is neither int nor float.")

(* val codegen : exp -> 'a codevalue environment 
     -> image -> 'a codevalue *)
let rec codegen expr env source  =
  let (Node (dtype,op,args)) = expr in
  let subeval x = codegen x env source in
  let arec = List.map subeval in
  let lift = function
    | VInt   i -> CInt   .< i >.
    | VFloat f -> CFloat .< f >.
    | VBool  b -> CBool  .< b >.
    | VMatrix b -> CMatrix  .< b >. 
       (* Note: This match will never occur if expr was
            simplified by a previous run of Staticeval.simplify. *)
  in 
  match op with
  | C v -> lift v
  | V s -> lookup_excp s env 
  | Read color -> 
      let (row,col) =
        match head2 (arec args) with
        | (CInt row, CInt col) -> (row,col)
        | _ -> raise (Inter_error
        "in 'codegen': Non-int arg for color operator. Bug in parsing?")
      in
      let rast = source.raster in
      let pnt = .< rast.(.~row).(.~col) >. in
      let maxv = float_of_int (source.maxval) in
      let fv i = CFloat .< float_of_int .~i /. maxv >. in
      begin match color with
      | Red   -> fv .< (.~pnt).redChannel   >. 
      | Green -> fv .< (.~pnt).greenChannel >. 
      | Blue  -> fv .< (.~pnt).blueChannel  >. 
      end
  | Int2Float -> let a = unCInt (head (arec args)) in
                 CFloat .< float_of_int .~a >.
  | Floor     -> let a = unCFloat (head (arec args)) in
                 CInt .< int_of_float .~a >.
  | UnOp  op -> let a = head (arec args) 
                in unopfunCode op a  
  | BinOp op -> let (a,b) = head2 (arec args) 
                in binopfunCode op a b  
  | If -> 
     let (c,a,b) = head3 args in
     let (c',a',b') = head3 (arec args) in
     begin match (a',b') with
       | (CBool a,  CBool b)  -> CBool  .< if .~(unCBool c') then .~a else .~b >.
       | (CInt a,   CInt b)   -> CInt   .< if .~(unCBool c') then .~a else .~b >.
       | (CFloat a, CFloat b) -> CFloat .< if .~(unCBool c') then .~a else .~b >.
       | _ -> raise (Inter_error
         "in 'codegen': Invalid types for if-branches. Bug in parser?")
     end
  | Let s ->
     let (rhs, body) = head2 args in 
     let rhs' = codegen rhs env source in
     let body_gen b = 
       let env' = extEnv (s,b) env in
       codegen body env' source
     in
     begin match (dtypeof rhs,dtypeof body) with
       | (Int,Int)     -> CInt .< let b = .~(unCInt rhs') in
                           .~(unCInt (body_gen (CInt .<b>.))) >.
       | (Int,Float)   -> CFloat .< let b = .~(unCInt rhs') in
                            .~(unCFloat (body_gen (CInt .<b>.))) >.
       | (Float,Int)   -> CInt .< let b = .~(unCFloat rhs') in
                            .~(unCInt (body_gen (CFloat .<b>.))) >.
       | (Float,Float) -> CFloat .< let b = .~(unCFloat rhs') in
                            .~(unCFloat (body_gen (CFloat .<b>.))) >.
       | (Matrix,Int) ->
           CInt .< let b = .~(unCMatrix rhs') in
                   .~(unCInt (body_gen (CMatrix .<b>.))) >.
       | (Matrix,Float) ->
           CFloat .< let b = .~(unCMatrix rhs') in
                   .~(unCFloat (body_gen (CMatrix .<b>.))) >.
       | _ -> raise (Inter_error
         "in 'codegen': Invalid types for 'let'. Bug in parser?")
   end
  | Sum s ->
     let (lb,ub,body) = head3 args in
     let lb' = unCInt (codegen lb env source) 
     and ub' = unCInt (codegen ub env source) in
     let iteration i = 
       let env' = extEnv (s,CInt i) env in
       codegen body env' source 
     in
     let range = .<fromto .~lb' .~ub'>. in
     begin match dtype with
     | Int ->
         let addi = .< fun n i -> n + .~(unCInt (iteration .<i>.)) >. in 
         CInt .< List.fold_left .~addi 0 .~range >.
     | Float ->
         let addi = .< fun n i -> n +. .~(unCFloat (iteration .<i>.)) >. in
         CFloat .< List.fold_left .~addi 0.0 .~range >.
       | _ -> raise (Inter_error
         "in 'codegen': Invalid type for 'sum'. Bug in parser?")
     end 
 | IndexMatrix s ->
     let x,y = match head2 (arec args) with
     | CInt x, CInt y -> x, y
     | _ -> raise (Inter_error
         "in 'codegen': Invalid type for matrix indexes. Bug in parser?")
     in
     begin match lookup_excp s env with
     | CMatrix m ->
         CFloat .< (.~m).(.~x).(.~y) >.
     | _ -> raise (Inter_error ("variable '"^s^
                   "' is not of type matrix"))
     end



let parse funString = 
  Filt_parser.main Filt_lexer.token (Lexing.from_string funString)

let interp env exp image = 
  let scale_to_int v = 
    let maxv = image.maxval in
    let maxvf = float_of_int maxv in
    .< let c = int_of_float (.~v *. maxvf ) in
       if c<0 then 0 else if c >maxv then maxv else c >.
  in
  let denv row col =  
    extEnv ("row", CInt row)
    (extEnv ("col", CInt col) env) 
  in 
  .< 
  fun (row,col) ->
    .~(scale_to_int (saveUnCFloat (codegen exp (denv .<row>. .<col>.) image)))
  >.                     

let compile exp image = .!(interp emptyEnv exp image)
let compile_withstaticvars exp image = 
  let env =  
    extEnv ("maxval", CInt .< image.maxval >.) emptyEnv 
  in 
  .!(interp env exp image)
