(*

  application: image filtering
  state: experimental
  revision: $Id: test_inter.ml,v 1.11 2004/11/20 14:25:35 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

open Defs
open Ppm

let img = Ppm.create 10 10 255 Ppm.black
let gradient="0.5 + let m=[-1.0 0.0 1.0 "
       ^ "            |-1.0 0.0 1.0 "
       ^ "            |-1.0 0.0 1.0] "
       ^ "  in sum i from 0 to 2 of "
       ^ "      sum j from 0 to 2 of "
       ^ "       m[i,j] * red(row-i-1,col-j-1)"
let step_inter="let r = 425+row/20.0 in "
      ^ "let c = 294+col/20.0 in "
      ^ "let lowr=floor(r) in "
      ^ "let lowc=floor(c) in "
      ^ "red(lowr,lowc)"

let bilinear_inter="let r = 425+row/20.0 in "
          ^ "let c = 294+col/20.0 in "
          ^ "let lowr=floor(r) in "
          ^ "let hir=lowr + 1 in "
          ^ "let lowc=floor(c) in "
          ^ "let hic=lowc + 1 in "
          ^ "(if c-lowc< r-lowr then "
          ^ "  (blue(hir,lowc)-blue(lowr,lowc))*(r-lowr) + "
          ^ "  (blue(hir,hic)-blue(hir,lowc))*(c-lowc)  "
          ^ "else "
          ^ "  (blue(hir,hic)-blue(lowr,hic))*(r-lowr) + "
          ^ "  (blue(lowr,hic)-blue(lowr,lowc))*(c-lowc) ) "
          ^ " + blue(lowr,lowc) "

let testcases = [
(*  "1.0";
  "1";
  "col";
  "row";
  "1.0*col";
  "2.0*4";
  "let i=1 in i";
  "let i=1 in 3.0";
  "let i=col in 3.0";
  "let i=col in i*i";
  "let i=7 in i*i*i";
  "let i=7 in sum j from 0 to 4 of i";
  "let a=[ 1.0 2.0 ] in a[0,0]";
  "let a=[ 1.0 2.0 3.0 ] in sum i from 0 to 2 of a[0,i]"
  *)
  gradient;
  step_inter;
  bilinear_inter
]


      let parse formulae = begin
  let lexbuf = Lexing.from_string formulae in
  let res = ref [] in
  begin try
    while true do
      res := 
        Filt_parser.main Filt_lexer.token lexbuf::!res
    done
  with 
  | Parsing.Parse_error ->
    begin
      let pos = lexbuf.Lexing.lex_start_p.Lexing.pos_cnum in
      prerr_string "Parse error in the following expression:\n\n";
      prerr_string lexbuf.Lexing.lex_buffer;
      prerr_string "\nUnexpected character '";
      prerr_char lexbuf.Lexing.lex_buffer.[pos];
      prerr_string "' at position ";
      prerr_int pos;
      prerr_string "!\n";
      prerr_string "\nexiting.\n";
      flush stderr;
      raise Parsing.Parse_error; 
    end
  | Filt_lexer.Eof -> ();
  end;
  !res
end
let simplify ?(width=100) ?(height=200) exp = 
  Staticeval.simplify exp width height 1 
let stagedinter exp = Stagedinter.interp emptyEnv exp img
let all s =
  begin
    let e = List.hd (parse s) in
    let e'= simplify ~width:img.width ~height:img.height  e in
    (s,stagedinter e')
  end
let all_testcases () = List.map all testcases
